/*
    TYPE_NAME ENV_1, ENV_2 ... ENV_n
    Type_1    -      -         -
    Type_2    -      -         -
    Type_3    -      -         -
 */

import 'package:dartrofit/src/strings.dart';

class BaseUrlManager {
  final container = <String, Map<String, String>>{};

  String get(String type, String env) {
    if (!Strings.isNullOrEmpty(type) &&
        !Strings.isNullOrEmpty(env) &&
        container.containsKey(type)) {
      final envUrl = container[type];
      if (envUrl != null && envUrl.containsKey(env)) {
        return envUrl[env];
      }
    }
    return null;
  }

  void put(String type, String env, String url) {
    if (Strings.isNullOrEmpty(type) ||
        Strings.isNullOrEmpty(env) ||
        Strings.isNullOrEmpty(url)) {
      throw Error();
    }

    container.putIfAbsent(type, () => <String, String>{})[env] = url;
  }

  void configure(void Function(BaseUrlConfiguration) configure) {
    final configuration = BaseUrlConfiguration(this);
    configure(configuration);
  }
}

class BaseUrlConfiguration {
  final BaseUrlManager baseUrl;

  BaseUrlConfiguration(this.baseUrl);

  void create(String type, void Function(EnvUrlConfiguration) configure) {
    final configuration = EnvUrlConfiguration();
    configure(configuration);
    configuration.envUrl.forEach((env, url) {
      baseUrl.put(type, env, url);
    });
  }
}

class EnvUrlConfiguration {
  final Map<String, String> envUrl = <String, String>{};

  void create(String env, String url) {
    envUrl[env] = url;
  }
}
