import 'dart:convert' as dart_convert;

import 'package:dartrofit/src/converter.dart';
import 'package:dartrofit/src/response_body.dart';
import 'package:dartrofit/src/type_value.dart';

class BuiltInConvertFactory extends ConverterFactory {
  @override
  Converter<ResponseBody, T> responseBodyConverter<T>(TypeValue returnType) {
    if (returnType is ParameterizedTypeValue) {
      final upperBoundAtIndex0 = returnType.upperBoundAtIndex0;
      TypeValue dataType;
      if (TypeValue.isDartrofitResponse(upperBoundAtIndex0)) {
        dataType =
            (upperBoundAtIndex0 as ParameterizedTypeValue).upperBoundAtIndex0;
      } else {
        dataType = upperBoundAtIndex0;
      }
      if (_isDartrofitResponseBody(dataType)) {
        return ResponseBodyConverter() as Converter<ResponseBody, T>;
      }

      if (_isDartCoreMap(dataType)) {
        try {
          return JsonConverter() as Converter<ResponseBody, T>;
        } catch (e) {
          throw UnsupportedError('You might want to convert to JSON, use Map<String, dynamic> instead');
        }
      }
    }
    return null;
  }

  static bool _isDartrofitResponseBody(TypeValue typeValue) {
    return typeValue.name == 'ResponseBody' &&
        typeValue.libraryName == 'dartrofit';
  }

  static bool _isDartCoreMap(TypeValue typeValue) {
    return typeValue.name == 'Map' && typeValue.libraryName == 'dart.core';
  }
}

class JsonConverter implements Converter<ResponseBody, Map<String, dynamic>> {
  @override
  Map<String, dynamic> convert(ResponseBody value) {
    return dart_convert.jsonDecode(value.string) as Map<String, dynamic>;
  }
}

class ResponseBodyConverter implements Converter<ResponseBody, ResponseBody> {
  @override
  ResponseBody convert(ResponseBody value) => value;
}
