import 'dart:convert';

import 'package:dartrofit/src/call_adapter.dart';
import 'package:dartrofit/src/converter.dart';
import 'package:dartrofit/src/request_factory.dart';
import 'package:dartrofit/src/response_body.dart';
import 'package:dartrofit/src/type_value.dart';
import 'package:dartrofit/dartrofit.dart';

class Dartrofit {
  final String baseUrl;
  final List<ConverterFactory> converterFactories = <ConverterFactory>[
    BuiltInConvertFactory()
  ];
  final List<CallAdapterFactory> adapterFactories = <CallAdapterFactory>[
    BuiltInAdapterFactory()
  ];

  Dartrofit(this.baseUrl) : assert(baseUrl != null);

  HttpServiceMethod<ResponseT, ReturnT> parseAnnotation<ResponseT, ReturnT>(
      MethodInfo methodInfo) {
    final requestFactory = RequestFactory(
      methodInfo.httpMethod,
      baseUrl,
      methodInfo.relativeUrl,
      methodInfo.isMultipart,
      methodInfo.parameterHandlers,
    );
    return _callAdapted<ResponseT, ReturnT>(
        methodInfo.returnType, requestFactory);
  }

  HttpServiceMethod<ResponseT, ReturnT> _callAdapted<ResponseT, ReturnT>(
      TypeValue returnType, RequestFactory requestFactory) {
    final adapter = _nextAdapter<ResponseT, ReturnT>(returnType);
    final converter = _nextConverter<ResponseT>(returnType);
    return CallAdapted<ResponseT, ReturnT>(requestFactory, adapter, converter);
  }

  CallAdapter<ResponseT, ReturnT> _nextAdapter<ResponseT, ReturnT>(
      TypeValue returnType) {
    for (var factory in adapterFactories) {
      final adapter = factory.get<ResponseT, ReturnT>(returnType);
      if (adapter != null) {
        return adapter;
      }
    }
    return null;
  }

  Converter<ResponseBody, ResponseT> _nextConverter<ResponseT>(
      TypeValue returnType) {
    for (var factory in converterFactories) {
      final converter = factory.responseBodyConverter<ResponseT>(returnType);
      if (converter != null) {
        return converter;
      }
    }
    return null;
  }
}
