import 'dart:collection';
import 'dart:convert';

import 'package:http/http.dart';
import 'package:dartrofit/src/maps.dart';
import 'package:dartrofit/src/strings.dart';

class RequestBuilder {
  final String method;
  Uri uri;
  final bool isMultipart;
  final Map<String, String> headers = <String, String>{};

  /// Maybe [String], [List<int>] or [Map<String, String]
  dynamic body;
  Encoding encoding;

  final multipartFields = <String, String>{};
  final multipartFiles = <MultipartFile>[];

  RequestBuilder(
      this.method, String baseUrl, String relativeUrl, this.isMultipart,
      {Map<String, String> headers})
      : uri = _parseInternal(baseUrl, relativeUrl) {
    if (!headers.isNullOrEmpty()) {
      this.headers.addAll(headers);
    }
  }

  static Uri _parseInternal(String baseUrl, String relativeUrl) {
    // check something?
    return Uri.parse(baseUrl + relativeUrl);
  }

  void addQueryParam(String name, String value) {
    if (Strings.isNullOrEmpty(name) || Strings.isNullOrEmpty(value)) {
      return;
    }
    final newQueryParameters = LinkedHashMap.of(uri.queryParameters)
      ..[name] = value;
    uri = uri.replace(queryParameters: newQueryParameters);
  }

  /// one of [Request], [MultipartRequest]
  BaseRequest build() {
    BaseRequest baseRequest;
    if (isMultipart) {
      final multipartRequest = MultipartRequest(method, uri);
      if (multipartFields.isNotEmpty) multipartRequest.fields.addAll(multipartFields);
      if (multipartFiles.isNotEmpty) multipartRequest.files.addAll(multipartFiles);
      baseRequest = multipartRequest;
    } else {
      final request = Request(method, uri);
      if (encoding != null) request.encoding = encoding;
      if (body != null) {
        if (body is String) {
          request.body = body as String;
        } else if (body is List) {
          request.bodyBytes = body.cast<int>() as List<int>;
        } else if (body is Map) {
          request.bodyFields =
              body.cast<String, String>() as Map<String, String>;
        } else {
          throw ArgumentError('Invalid request body "$body".');
        }
      }
      baseRequest = request;
    }
    if (!headers.isNullOrEmpty()) baseRequest.headers.addAll(headers);
    return baseRequest;
  }
}
