import 'package:dartrofit/src/parameter_handler.dart';
import 'package:dartrofit/src/request_builder.dart';
import 'package:http/http.dart';

class RequestFactory {
  final String method;
  final String baseUrl;
  final String relativeUrl;
  final bool isMultipart;
  final List<ParameterHandler<Object>> parameterHandlers;

  RequestFactory(this.method, this.baseUrl, this.relativeUrl, this.isMultipart, this.parameterHandlers);

  /// one of [Request], [MultipartRequest] and [StreamedRequest]
  BaseRequest create(List<Object> args) {
    final handlers = parameterHandlers;
    final argumentCount = args.length;
    if (argumentCount != handlers.length) {
      throw ArgumentError(
          "Argument count ($argumentCount) doesn't match expected count (${handlers.length})");
    }
    final requestBuilder = RequestBuilder(method, baseUrl, relativeUrl, isMultipart);

    for (var p = 0; p < argumentCount; p++) {
      handlers[p].apply(requestBuilder, args[p]);
    }

    return requestBuilder.build();
  }
}
